/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * EventHistory.cpp
 *
 *  Created on: 2011/01/24
 *      Author: HARK Development Team
 */

#include <stdexcept>
#include <EventHistory.hpp>
#include <Timeval.hpp>

using namespace std;

namespace NSHarkBinaural
{

EventHistory::EventHistory(EventID eid, const timeval& currentTime, const timeval& validPausePeriod,
                           Band fundamental, NSDimensionPromise::Angle direction, double updateRatioThreshold)
    : eid(eid), status(EventStatus::ONSET), fundamental(fundamental), direction(direction),
      pauseTime(), durationTime(), onsetTime(currentTime), validPausePeriod(validPausePeriod),
      updateCount(1), updateRatioThreshold(updateRatioThreshold),
      logger(Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME))
{
    timeval zero = { 0, 0 };
    if (validPausePeriod < zero)
    {
        throw out_of_range("valid pause period is minus");
    }
}

void EventHistory::Update(const timeval& currentTime, Band fundamental, NSDimensionPromise::Angle direction)
{
    this->status = EventStatus::DURATION;
    this->fundamental = fundamental;
    this->direction = direction;
    this->durationTime = currentTime;
    this->updateCount++;
    timerclear(&this->pauseTime);

    // 更新頻度チェック
    CheckUpdateRatio(currentTime);
}

void EventHistory::Update(const timeval& currentTime)
{
    if (this->status == EventStatus::ONSET || this->status == EventStatus::DURATION)
    {
        this->status = EventStatus::PAUSE;
        this->pauseTime = currentTime;
    }
    else if (this->status == EventStatus::PAUSE)
    {
        timeval invalidTime; // イベントが無効となる時刻
        invalidTime = this->pauseTime + this->validPausePeriod;
        if (currentTime > invalidTime)
        {
            // イベントは終了
            this->status = EventStatus::OFFSET;
        }
    }
    // 更新頻度チェック
    CheckUpdateRatio(currentTime);
}

void EventHistory::CheckUpdateRatio(const timeval& currentTime)
{
    if (this->status != EventStatus::OFFSET)
    {
        double time = TimevalToSeconds(currentTime - this->onsetTime);
        double ratio = this->updateCount / time;
        if (time > 0 && this->updateCount > 0 && ratio <= this->updateRatioThreshold)
        {
            LogFinest(logger, boost::format("update ratio = %lf (%d / %lf), so this event will be offset")
                      % ratio % this->updateCount % time);
            this->status = EventStatus::OFFSET;
        }
    }
}


}
